package org.apache.maven.surefire.report;

import java.util.concurrent.Callable;

/*
* Licensed to the Apache Software Foundation (ASF) under one
* or more contributor license agreements.  See the NOTICE file
* distributed with this work for additional information
* regarding copyright ownership.  The ASF licenses this file
* to you under the Apache License, Version 2.0 (the
* "License"); you may not use this file except in compliance
* with the License.  You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing,
* software distributed under the License is distributed on an
* "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
* KIND, either express or implied.  See the License for the
* specific language governing permissions and limitations
* under the License.
*/
class RunnableTestClass2
    implements Callable<Object>

{

    @Override
    public Object call()
        throws Exception
    {
        InnerRunnableTestClass.cThrows();
        return null;  //To change body of implemented methods use File | Settings | File Templates.
    }

    static class InnerRunnableTestClass
    {
        public static void cThrows()
            throws Exception
        {
            throw new Exception( "Hey ho, hey ho, a throwable we throw!" );
        }
    }
}
