package org.apache.maven.surefire.junitcore.pc;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.junit.runner.Description;

import java.util.Collection;
import java.util.Collections;

/**
 * Populates collection {@code triggeredTests} of descriptions started before shutting down.
 * Populates collection {@code incompleteTests} which describes started tests but unfinished due to abrupt shutdown.
 * The collection {@code triggeredTests} contains all elements from {@code incompleteTests}.
 *
 * @author <a href="mailto:tibordigana@apache.org">Tibor Digana (tibor17)</a>
 * @see Scheduler
 * @since 2.18
 */
public final class ShutdownResult
{
    private final Collection<Description> triggeredTests;

    private final Collection<Description> incompleteTests;

    public ShutdownResult( Collection<Description> triggeredTests, Collection<Description> incompleteTests )
    {
        this.triggeredTests = triggeredTests == null ? Collections.<Description>emptySet() : triggeredTests;
        this.incompleteTests = incompleteTests == null ? Collections.<Description>emptySet() : incompleteTests;
    }

    public Collection<Description> getTriggeredTests()
    {
        return triggeredTests;
    }

    public Collection<Description> getIncompleteTests()
    {
        return incompleteTests;
    }
}
