package org.apache.maven.surefire.its.jiras;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.apache.maven.surefire.its.fixture.SurefireJUnit4IntegrationTestCase;
import org.apache.maven.surefire.its.fixture.SurefireLauncher;
import org.junit.Test;

/**
 * "verify" goal ignores "dependenciesToScan" parameter when checking tests existence
 * <p>
 * Found in Surefire 2.16.
 *
 * @author <a href="mailto:tibordigana@apache.org">Tibor Digana (tibor17)</a>
 * @see <a href="https://issues.apache.org/jira/browse/SUREFIRE-1024">SUREFIRE-1024</a>
 * @since 2.19
 */
public class Surefire1024VerifyFailsafeIfTestedIT
    extends SurefireJUnit4IntegrationTestCase
{

    @Test
    public void shouldScanAndRunTestsInDependencyJars() throws Exception {
        SurefireLauncher launcher = unpack( "surefire-1024" );
        launcher.executeVerify()
            .verifyTextInLog( "class jiras.surefire1024.A1IT#test() dependency to scan" );

        launcher.getSubProjectValidator( "jiras-surefire-1024-it" )
            .assertIntegrationTestSuiteResults( 1, 0, 0, 0 );
    }
}
