package org.apache.maven.surefire.report;

import junit.framework.TestCase;

/*
 * Copyright 2002-2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

public class RunStatisticsTest
    extends TestCase
{
    public void testSetRunStatistics()
    {
        RunStatistics statistics = new RunStatistics();
        statistics.set( 10, 5, 2, 1, 2 );
        assertEquals( 10, statistics.getCompletedCount() );
        assertEquals( 5, statistics.getErrors() );
        assertEquals( 2, statistics.getFailures() );
        assertEquals( 1, statistics.getSkipped() );
        assertEquals( 2, statistics.getFlakes() );
    }
}
