package org.apache.maven.surefire.its.jiras;
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.apache.maven.surefire.its.fixture.SurefireJUnit4IntegrationTestCase;

import org.junit.Ignore;
import org.junit.Test;

public class Surefire806SpecifiedTestControlsIT
    extends SurefireJUnit4IntegrationTestCase
{

    @Test
    @Ignore( "since SUREFIRE-1153 the includes/excludes are overridden by -Dtest or it.test for whatever execution" )
    public void singleTestInOneExecutionOfMultiExecutionProject()
    {
        unpack( "/surefire-806-specifiedTests-multi" ).setTestToRun( "FirstTest" ).failIfNoSpecifiedTests(
            false ).executeTest().verifyErrorFree( 1 );
    }

    @Test
    @Ignore( "since SUREFIRE-1153 the includes/excludes are overridden by -Dtest or it.test for whatever execution" )
    public void twoSpecifiedTestExecutionsInCorrectExecutionBlocks()
    {
        unpack( "/surefire-806-specifiedTests-multi" ).setTestToRun(
            "FirstTest,SecondTest" ).executeTest().verifyErrorFree( 2 );
    }

    @Test
    public void singleTestInSingleExecutionProject()
    {
        unpack( "/surefire-806-specifiedTests-single" ).setTestToRun( "ThirdTest" ).failIfNoSpecifiedTests(
            false ).executeTest().verifyErrorFree( 1 );
    }
}
