package foo;

import org.junit.Test;

public class Methods6Test
{
    @Test
    public void first()
    {
        SleepUtil.sleep();
    }

    @Test
    public void second()
    {
        SleepUtil.sleep();
    }

    @Test
    public void third()
    {
        SleepUtil.sleep();
    }

    @Test
    public void fourth()
    {
        SleepUtil.sleep();
    }

    @Test
    public void fifth()
    {
        SleepUtil.sleep();
    }

    @Test
    public void sixth()
    {
        SleepUtil.sleep();
    }
}
